"""
sciduck

single cell data analyis unification and communication toolkit
"""

__version__ = "0.4.0"
__authors__ = ['Nelson Johansen', 'Matthew Schmitz', 'Dan Yuan']
__credits__ = 'Allen Institute for Brain Science'

from . import basic_qc as qc
from . import plotting as pl
from . import annotation_table as anno
from . import entropy_qc as eq
from . import templates as template
